@@ -16,13 +16,11 @@ from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Generator
 from typing import Literal
-from typing import Optional
-from typing import Union
 
 import logging
 import time
+from collections.abc import Generator
 from datetime import datetime
 from datetime import timedelta
 
@@ -52,6 +50,7 @@ from gajim.gtk.conversation.rows.base import BaseRow
 from gajim.gtk.conversation.rows.call import CallRow
 from gajim.gtk.conversation.rows.command_output import CommandOutputRow
 from gajim.gtk.conversation.rows.date import DateRow
+from gajim.gtk.conversation.rows.encryption_info import EncryptionInfoRow
 from gajim.gtk.conversation.rows.file_transfer import FileTransferRow
 from gajim.gtk.conversation.rows.file_transfer_jingle import \
     FileTransferJingleRow
@@ -106,7 +105,7 @@ class ConversationView(Gtk.ScrolledWindow):
         self._list_box.set_selection_mode(Gtk.SelectionMode.NONE)
         self._list_box.set_sort_func(self._sort_func)
 
-        self._contact: Optional[ChatContactT] = None
+        self._contact: ChatContactT | None = None
         self._client = None
 
         # Keeps track of the number of rows shown in ConversationView
@@ -124,10 +123,10 @@ class ConversationView(Gtk.ScrolledWindow):
 
         self._current_upper: float = 0
         self._autoscroll: bool = True
-        self._request_history_at_upper: Optional[float] = None
+        self._request_history_at_upper: float | None = None
         self._upper_complete: bool = False
         self._lower_complete: bool = True
-        self._requesting: Optional[str] = None
+        self._requesting: str | None = None
         self._block_signals = False
 
         self._signal_handlers_enabled = False
@@ -156,7 +155,7 @@ class ConversationView(Gtk.ScrolledWindow):
         self.disable_row_selection()
 
     def enable_row_selection(self,
-                             log_line_id: Optional[int]
+                             log_line_id: int | None
                              ) -> None:
 
         self._list_box.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
@@ -243,7 +242,6 @@ class ConversationView(Gtk.ScrolledWindow):
 
         self._block_signals = False
         self._enable_signal_handlers(True)
-        self._emit('request-history', True)
 
     def get_autoscroll(self) -> bool:
         return self._autoscroll
@@ -372,15 +370,17 @@ class ConversationView(Gtk.ScrolledWindow):
         return self._contact
 
     def _get_row_at_index(self, index: int) -> BaseRow:
-        return cast(BaseRow, self._list_box.get_row_at_index(index))
+        row = self._list_box.get_row_at_index(index)
+        assert row is not None
+        return cast(BaseRow, row)
 
-    def get_first_message_row(self) -> Optional[MessageRow]:
+    def get_first_message_row(self) -> MessageRow | None:
         for row in self._list_box.get_children():
             if isinstance(row, MessageRow):
                 return row
         return None
 
-    def get_last_message_row(self) -> Optional[MessageRow]:
+    def get_last_message_row(self) -> MessageRow | None:
         children = self._list_box.get_children()
         children.reverse()
         for row in children:
@@ -388,17 +388,17 @@ class ConversationView(Gtk.ScrolledWindow):
                 return row
         return None
 
-    def get_first_event_row(self) -> Optional[Union[InfoMessage, MUCJoinLeft]]:
+    def get_first_event_row(self) -> InfoMessage | MUCJoinLeft | None:
         for row in self._list_box.get_children():
-            if isinstance(row, (InfoMessage, MUCJoinLeft)):
+            if isinstance(row, InfoMessage | MUCJoinLeft):
                 return row
         return None
 
-    def get_last_event_row(self) -> Optional[Union[InfoMessage, MUCJoinLeft]]:
+    def get_last_event_row(self) -> InfoMessage | MUCJoinLeft | None:
         children = self._list_box.get_children()
         children.reverse()
         for row in children:
-            if isinstance(row, (InfoMessage, MUCJoinLeft)):
+            if isinstance(row, InfoMessage | MUCJoinLeft):
                 return row
         return None
 
@@ -410,7 +410,7 @@ class ConversationView(Gtk.ScrolledWindow):
 
     def add_muc_subject(self,
                         subject: MucSubject,
-                        timestamp: Optional[float] = None
+                        timestamp: float | None = None
                         ) -> None:
 
         muc_subject = MUCSubject(self.contact.account, subject, timestamp)
@@ -448,7 +448,7 @@ class ConversationView(Gtk.ScrolledWindow):
 
     def add_info_message(self,
                          text: str,
-                         timestamp: Optional[float] = None
+                         timestamp: float | None = None
                          ) -> None:
 
         message = InfoMessage(self.contact.account, text, timestamp)
@@ -458,13 +458,13 @@ class ConversationView(Gtk.ScrolledWindow):
         transfer_row = FileTransferRow(self.contact.account, transfer)
         self._insert_message(transfer_row)
 
-    def add_jingle_file_transfer(self,
-                                 event: Union[
-                                     events.FileRequestReceivedEvent,
-                                     events.FileRequestSent,
-                                     None] = None,
-                                 db_message: Optional[ConversationRow] = None
-                                 ) -> None:
+    def add_jingle_file_transfer(
+        self,
+        event: (events.FileRequestReceivedEvent |
+                events.FileRequestSent |
+                None) = None,
+        db_message: ConversationRow | None = None
+    ) -> None:
 
         assert isinstance(self._contact, BareContact)
         jingle_transfer_row = FileTransferJingleRow(
@@ -474,9 +474,13 @@ class ConversationView(Gtk.ScrolledWindow):
             db_message=db_message)
         self._insert_message(jingle_transfer_row)
 
+    def add_encryption_info(self, event: events.EncryptionInfo) -> None:
+        assert self._contact is not None
+        self._insert_message(EncryptionInfoRow(event))
+
     def add_call_message(self,
-                         event: Optional[events.JingleRequestReceived] = None,
-                         db_message: Optional[ConversationRow] = None
+                         event: events.JingleRequestReceived | None = None,
+                         db_message: ConversationRow | None = None
                          ) -> None:
         assert isinstance(self._contact, BareContact)
         call_row = CallRow(
@@ -496,13 +500,13 @@ class ConversationView(Gtk.ScrolledWindow):
                     kind: str,
                     name: str,
                     timestamp: float,
-                    log_line_id: Optional[int] = None,
-                    message_id: Optional[str] = None,
-                    stanza_id: Optional[str] = None,
-                    display_marking: Optional[Displaymarking] = None,
-                    additional_data: Optional[AdditionalDataDict] = None,
-                    marker: Optional[str] = None,
-                    error: Union[CommonError, StanzaError, None] = None
+                    log_line_id: int | None = None,
+                    message_id: str | None = None,
+                    stanza_id: str | None = None,
+                    display_marking: Displaymarking | None = None,
+                    additional_data: AdditionalDataDict | None = None,
+                    marker: str | None = None,
+                    error: CommonError | StanzaError | None = None
                     ) -> None:
 
         if not timestamp:
@@ -555,7 +559,7 @@ class ConversationView(Gtk.ScrolledWindow):
         self._active_date_rows.add(start_of_day)
         self._list_box.add(date_row)
 
-        row = self._get_row_at_index(date_row.get_index() + 1)
+        row = self._list_box.get_row_at_index(date_row.get_index() + 1)
         if row is None:
             return
 
@@ -578,11 +582,11 @@ class ConversationView(Gtk.ScrolledWindow):
             if message.is_mergeable(ancestor):
                 message.set_merged(True)
 
-    def _find_ancestor(self, message: MessageRow) -> Optional[MessageRow]:
+    def _find_ancestor(self, message: MessageRow) -> MessageRow | None:
         index = message.get_index()
         while index != 0:
             index -= 1
-            row = self._get_row_at_index(index)
+            row = self._list_box.get_row_at_index(index)
             if row is None:
                 return None
 
@@ -603,7 +607,7 @@ class ConversationView(Gtk.ScrolledWindow):
         index = message.get_index()
         while True:
             index += 1
-            row = self._get_row_at_index(index)
+            row = self._list_box.get_row_at_index(index)
             if row is None:
                 return
 
@@ -662,6 +666,20 @@ class ConversationView(Gtk.ScrolledWindow):
         row = self._get_row_at_index(len(self._list_box.get_children()) - 1)
         row.destroy()
 
+    def remove_message(self, log_line_id: int) -> None:
+        row = self.get_row_by_log_line_id(log_line_id)
+        if row is None:
+            return
+
+        index = row.get_index()
+        row.destroy()
+        decendant_row = self._list_box.get_row_at_index(index)
+        if isinstance(decendant_row, MessageRow):
+            # Unset possible merged state if we delete a 'top level' message.
+            # Checks for same sender etc. are not necessary, since we simply
+            # unset merged state.
+            decendant_row.set_merged(False)
+
     def scroll_to_message_and_highlight(self, log_line_id: int) -> None:
         highlight_row = None
         for row in cast(list[BaseRow], self._list_box.get_children()):
@@ -680,10 +698,10 @@ class ConversationView(Gtk.ScrolledWindow):
         adj = self.get_vadjustment()
         adj.set_value(adj.get_upper() - adj.get_page_size())
 
-    def _get_row_by_message_id(self, id_: str) -> Optional[MessageRow]:
+    def _get_row_by_message_id(self, id_: str) -> MessageRow | None:
         return self._message_id_row_map.get(id_)
 
-    def get_row_by_log_line_id(self, log_line_id: int) -> Optional[MessageRow]:
+    def get_row_by_log_line_id(self, log_line_id: int) -> MessageRow | None:
         for row in cast(list[BaseRow], self._list_box.get_children()):
             if not isinstance(row, MessageRow):
                 continue
@@ -691,7 +709,7 @@ class ConversationView(Gtk.ScrolledWindow):
                 return row
         return None
 
-    def get_row_by_stanza_id(self, stanza_id: str) -> Optional[MessageRow]:
+    def get_row_by_stanza_id(self, stanza_id: str) -> MessageRow | None:
         for row in cast(list[BaseRow], self._list_box.get_children()):
             if not isinstance(row, MessageRow):
                 continue
@@ -713,9 +731,6 @@ class ConversationView(Gtk.ScrolledWindow):
                 row.update()
 
     def set_read_marker(self, id_: str) -> None:
-        if id_ is None:
-            return
-
         row = self._get_row_by_message_id(id_)
         if row is None:
             return
@@ -736,7 +751,7 @@ class ConversationView(Gtk.ScrolledWindow):
     def correct_message(self,
                         correct_id: str,
                         text: str,
-                        nickname: Optional[str]
+                        nickname: str | None
                         ) -> None:
 
         message_row = self._get_row_by_message_id(correct_id)
@@ -763,8 +778,8 @@ class ConversationView(Gtk.ScrolledWindow):
     def _on_contact_setting_changed(self,
                                     value: Any,
                                     setting: str,
-                                    _account: Optional[str],
-                                    _jid: Optional[JID]) -> None:
+                                    _account: str | None,
+                                    _jid: JID | None) -> None:
 
         if setting == 'print_join_left':
             if value:
