@@ -15,13 +15,12 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Iterator
-from typing import Optional
 
 import itertools
 import logging
 import re
 import time
+from collections.abc import Iterator
 from datetime import datetime
 from datetime import timedelta
 
@@ -35,6 +34,10 @@ from gajim.common import ged
 from gajim.common.const import AvatarSize
 from gajim.common.const import Direction
 from gajim.common.const import KindConstant
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.modules.contacts import ResourceContact
 from gajim.common.storage.archive import SearchLogRow
 
 from gajim.gtk.builder import get_builder
@@ -57,13 +60,12 @@ class SearchView(Gtk.Box):
         Gtk.Box.__init__(self)
         self.set_size_request(300, -1)
 
-        self._account: Optional[str] = None
-        self._jid: Optional[JID] = None
-        self._results_iterator: Optional[Iterator[SearchLogRow]] = None
-        self._scope: str = 'everywhere'
+        self._account: str | None = None
+        self._jid: JID | None = None
+        self._results_iterator: Iterator[SearchLogRow] | None = None
 
-        self._first_date: Optional[datetime] = None
-        self._last_date: Optional[datetime] = None
+        self._first_date: datetime | None = None
+        self._last_date: datetime | None = None
 
         self._ui = get_builder('search_view.ui')
         self._ui.results_listbox.set_header_func(self._header_func)
@@ -83,7 +85,7 @@ class SearchView(Gtk.Box):
         self._clear()
 
     @staticmethod
-    def _header_func(row: ResultRow, before: ResultRow) -> None:
+    def _header_func(row: ResultRow, before: ResultRow | None) -> None:
         if before is None:
             row.set_header(RowHeader(row.account, row.jid, row.time))
         else:
@@ -155,16 +157,17 @@ class SearchView(Gtk.Box):
 
         everywhere = self._ui.search_checkbutton.get_active()
         context = self._account is not None and self._jid is not None
+        if not context:
+            # Started search without context -> show in UI
+            self._ui.search_checkbutton.set_active(True)
 
         if not context or everywhere:
-            self._scope = 'everywhere'
             self._results_iterator = app.storage.archive.search_all_logs(
                 text,
                 from_users=from_filters,
                 before=before_filters,
                 after=after_filters)
         else:
-            self._scope = 'contact'
             self._results_iterator = app.storage.archive.search_log(
                 self._account,
                 self._jid,
@@ -177,7 +180,7 @@ class SearchView(Gtk.Box):
 
     @staticmethod
     def _strip_filters(text: str,
-                       filter_name: str) -> tuple[str, Optional[list[str]]]:
+                       filter_name: str) -> tuple[str, list[str] | None]:
         filters: list[str] = []
         start = 0
         new_text = ''
@@ -193,18 +196,13 @@ class SearchView(Gtk.Box):
         accounts = self._get_accounts()
         assert self._results_iterator is not None
         for msg in itertools.islice(self._results_iterator, 25):
-            if self._scope == 'everywhere':
-                archive_jid = app.storage.archive.get_jid_from_id(msg.jid_id)
-                if archive_jid is None:
-                    continue
-                result_row = ResultRow(
-                    msg,
-                    accounts[msg.account_id],
-                    JID.from_string(archive_jid.jid))
-            else:
-                assert self._account is not None
-                assert self._jid is not None
-                result_row = ResultRow(msg, self._account, self._jid)
+            archive_jid = app.storage.archive.get_jid_from_id(msg.jid_id)
+            if archive_jid is None:
+                continue
+            result_row = ResultRow(
+                msg,
+                accounts[msg.account_id],
+                JID.from_string(archive_jid.jid))
 
             self._ui.results_listbox.add(result_row)
 
@@ -217,24 +215,24 @@ class SearchView(Gtk.Box):
         self._add_results()
 
     def _update_calendar(self) -> None:
-        if self._scope == 'everywhere':
+        if self._account is None and self._jid is None:
             self._ui.calendar_button.set_sensitive(False)
-        else:
-            self._ui.calendar_button.set_sensitive(True)
+            return
 
-            first_log = app.storage.archive.get_first_history_timestamp(
-                self._account, self._jid)
-            if first_log is None:
-                return
-            self._first_date = self._get_date_from_timestamp(first_log)
-            last_log = app.storage.archive.get_last_history_timestamp(
-                self._account, self._jid)
-            if last_log is None:
-                return
-            self._last_date = self._get_date_from_timestamp(last_log)
+        self._ui.calendar_button.set_sensitive(True)
 
-            month = gtk_month(self._last_date.month)
-            self._ui.calendar.select_month(month, self._last_date.year)
+        first_log = app.storage.archive.get_first_history_timestamp(
+            self._account, self._jid)
+        if first_log is None:
+            return
+        self._first_date = self._get_date_from_timestamp(first_log)
+        last_log = app.storage.archive.get_last_history_timestamp(
+            self._account, self._jid)
+        if last_log is None:
+            return
+        self._last_date = self._get_date_from_timestamp(last_log)
+        month = gtk_month(self._last_date.month)
+        self._ui.calendar.select_month(month, self._last_date.year)
 
     def _on_month_changed(self, calendar: Gtk.Calendar) -> None:
         # Mark days with history in calendar
@@ -358,16 +356,9 @@ class SearchView(Gtk.Box):
         self._clear()
         self._update_calendar()
 
-    def set_context(self, account: Optional[str], jid: Optional[JID]) -> None:
+    def set_context(self, account: str | None, jid: JID | None) -> None:
         self._account = account
         self._jid = jid
-        everywhere = bool(jid is None)
-        self._ui.search_checkbutton.set_active(everywhere)
-        context = account is not None and jid is not None
-        if not context or everywhere:
-            self._scope = 'everywhere'
-        else:
-            self._scope = 'contact'
         self._update_calendar()
 
 
@@ -381,6 +372,8 @@ class RowHeader(Gtk.Box):
 
         client = app.get_client(account)
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         self._ui.header_name_label.set_text(contact.name or '')
 
         local_time = time.localtime(timestamp)
@@ -410,6 +403,9 @@ class ResultRow(Gtk.ListBoxRow):
 
         self.contact = self._client.get_module('Contacts').get_contact(
             jid, groupchat=self.type == 'groupchat')
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
 
         self.get_style_context().add_class('search-view-row')
         self._ui = get_builder('search_view.ui')
@@ -445,10 +441,10 @@ class ResultRow(Gtk.ListBoxRow):
 
     def _get_avatar(self,
                     kind: str,
-                    name: str) -> Optional[cairo.ImageSurface]:
+                    name: str) -> cairo.ImageSurface | None:
 
         scale = self.get_scale_factor()
-        if self.contact.is_groupchat:
+        if isinstance(self.contact, GroupchatContact):
             contact = self.contact.get_resource(name)
             return contact.get_avatar(AvatarSize.ROSTER, scale, add_show=False)
 
@@ -458,4 +454,5 @@ class ResultRow(Gtk.ListBoxRow):
         else:
             contact = self.contact
 
+        assert not isinstance(contact, GroupchatContact | ResourceContact)
         return contact.get_avatar(AvatarSize.ROSTER, scale, add_show=False)
